// database row base class                                       -*- C++ -*-
// $Id: row_base.h,v 1.3 2004/01/18 00:11:16 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef PQXX_OBJECT_ROW_BASE_H
#define PQXX_OBJECT_ROW_BASE_H

#include <sigc++/object.h>
#include <sigc++/signal.h>

#include <pqxx-object/transaction.h>

namespace pqxxobject
{
  /**
   * Database row base class.
   */
  class row_base : public SigC::Object
  {
  public:
    enum row_state
      {
	STATE_UNINITIALISED,
	STATE_INITIALISED,
	STATE_INCONSISTENT
      };

    row_base();

    row_base(row_state status,
	     bool modified=false);

    row_base(const row_base& rhs);

    virtual ~row_base();


    row_base& operator = (const row_base& rhs);

    row_state get_state() const;

    bool is_modified() const;

    /**
     * Refresh a row from a table.
     * @param tran the transaction which will be used to execute queries.
     */
    void refresh(pqxxobject::transaction& tran);

    /**
     * Insert a row into a table.
     * @param tran the transaction which will be used to execute queries.
     */
    void insert(pqxxobject::transaction& tran);

    /**
     * Update a row from a table.
     * @param tran the transaction which will be used to execute queries.
     */
    void update(pqxxobject::transaction& tran);

    /**
     * Remove a row from a table.
     * @param tran the transaction which will be used to execute queries.
     */
    void erase(pqxxobject::transaction& tran);

    SigC::Signal0<void>& signal_changed();
    SigC::Signal0<void>& signal_refreshed();
    SigC::Signal0<void>& signal_inserted();
    SigC::Signal0<void>& signal_updated();
    SigC::Signal0<void>& signal_erased();

  protected:
    row_state m_state;
    bool m_modified;

    /**
     * Refresh a row from a table.
     * This default method does nothing and should be overridden by
     * derived classes.
     * @param tran the transaction which will be used to execute queries.
     */
    virtual void refresh_impl(pqxxobject::transaction& tran);

    /**
     * Insert a row into a table.
     * This default method does nothing and should be overridden by
     * derived classes.
     * @param tran the transaction which will be used to execute queries.
     */
    virtual void insert_impl(pqxxobject::transaction& tran);

    /**
     * Update a row from a table.
     * This default method does nothing and should be overridden by
     * derived classes.
     * @param tran the transaction which will be used to execute queries.
     */
    virtual void update_impl(pqxxobject::transaction& tran);

    /**
     * Remove a row from a table.
     * This default method does nothing and should be overridden by
     * derived classes.
     * @param tran the transaction which will be used to execute queries.
     */
    virtual void erase_impl(pqxxobject::transaction& tran);

    SigC::Signal0<void> m_signal_changed;
    SigC::Signal0<void> m_signal_refreshed;
    SigC::Signal0<void> m_signal_inserted;
    SigC::Signal0<void> m_signal_updated;
    SigC::Signal0<void> m_signal_erased;

  }; // class row_base

}; // namespace pqxxobject

#endif // PQXX_OBJECT_ROW_BASE_H
