// database table convenience wrapper                            -*- C++ -*-
// $Id: table.h,v 1.5 2004/01/17 23:30:44 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef PQXX_OBJECT_TABLE_H
#define PQXX_OBJECT_TABLE_H

#include <list>
#include <memory>

#include <pqxx-object/row.h>

namespace pqxxobject
{
  /**
   * Database table convenience wrapper.
   * This class provides methods to simplify table access for classes
   * which represent database tables.  Methods are provided to find
   * single and multiple rows and to insert, update and delete rows
   * from the table.  A default method for modifying the database is
   * also provided, but this should be overridden in all but the
   * simplest classes.
   */
  template<typename Row>
  class table
  {
  public:
    /// The type of the row the table contains.
    typedef Row row_type;
    typedef typename Row::row_ptr row_ptr;
    typedef std::list<Row> row_list;
    typedef std::auto_ptr<row_list> row_list_ptr;


  protected:
    /**
     * The constructor.
     * @param tran a pointer to a PostgreSQL transaction
     * object.  This must have been previously created from
     * connection.  If set to NULL, table will run in auto-commit
     * mode, and will create a new transaction for each database
     * operation.
     */
    table(pqxxobject::transaction& tran):
      m_transaction(tran)
    {}

  public:
    /// The destructor.
    virtual
    ~table()
    {}

    /**
     * Add a new row.
     * @param row the row to add.
     */
    virtual
    void
    insert(row_type& row)
    {
      row.insert(m_transaction);
    }

    /**
     * Update the details of an existing row.
     * @param row the row to update.
     */
    virtual
    void
    update(row_type& row)
    {
      row.update(m_transaction);
    }

    /**
     * Erase (remove) a row.
     * @param row the row to erase.
     */
    virtual
    void
    erase(row_type& row)
    {
      row.erase(m_transaction);
    }

    /**
     * Refresh a row (synchronise with current database state).
     * @param row the row to refresh.
     */
    virtual
    void
    refresh(row_type& row)
    {
      row.refresh(m_transaction);
    }

  protected:
    /**
     * Find a single row from the database.
     * @param query the SQL query to run.
     * @returns a pointer to row, or NULL if no member was found.
     */
    virtual
    row_ptr
    find_one(const std::string& query)
    {
      try
	{
	  m_transaction.begin("pqxxobject::table::find_one");

	  pqxx::result R (m_transaction.exec(query));


	  row_ptr ret;

	  if (R.size() > 0)
	    {
	      pqxx::result::const_iterator cur = R.begin();
	      // Assign result (if any) to temporary row.
	      if (cur != R.end())
		ret = row_type::create(cur, m_transaction);
	    }

	  m_transaction.end();
	  return ret;
	}
      catch (const std::exception& e)
	{
	  m_transaction.end();
	  throw DatabaseError(e.what());
	}
    }

    /**
     * Find multiple rows from the database.
     * @param query the SQL query to run.
     * @returns a pointer to a vector of rows.
     */
    virtual
    row_list_ptr
    find_many(const std::string& query)
    {
      try
	{
	  m_transaction.begin("pqxxobject::table::find_many");

	  pqxx::result R (m_transaction.exec(query));

	  row_list_ptr ret(new row_list);

	  for (pqxx::result::const_iterator cur = R.begin(); cur != R.end(); cur++)
	    {
	      row_ptr tmp_row;
	      // Assign results (one row) to temporary row
	      tmp_row = row_type::create(cur, m_transaction);
	      // Add this member to the list
	      ret->push_back(*tmp_row);
	    }

	  m_transaction.end();

	  return ret;
	}
      catch (const std::exception& e)
	{
	  m_transaction.end();
	  throw DatabaseError(e.what());
	}
    }

    pqxxobject::transaction& m_transaction;

  }; // class table

}; // namespace pqxxobject

#endif // PQXX_OBJECT_TABLE_H
