// database transaction convenience wrapper                      -*- C++ -*-
// $Id: transaction.cc,v 1.5 2004/01/17 23:30:44 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <cassert>

#include <sigc++/object.h>

#include <pqxx-object/transaction.h>

using namespace pqxxobject;

transaction::transaction(pqxx::connection& connection,
			 pqxx::transaction<>*& transaction):
  m_connection(connection),
  m_transaction(transaction),
  m_autocommit(true),
  m_state(STATE_NONE)
{
  // Set up database.
  // Turn off auto-committing of changes if the creator gave us a transaction.
  if (m_transaction != NULL)
    m_autocommit = false;
}

transaction::~transaction()
{
}

SigC::Signal0<void>&
transaction::signal_commit()
{
  return m_signal_commit;
}

SigC::Signal0<void>&
transaction::signal_abort()
{
  return m_signal_abort;
}

void
transaction::begin(const std::string& name)
{
  assert ((m_transaction == NULL && m_autocommit == true) ||
	  (m_transaction != NULL && m_autocommit == false));

  try
    {
      if (m_transaction == NULL)
	{
	  assert (m_state != STATE_EXECUTING); // can't be executing without a transaction
	  m_transaction = new pqxx::transaction<> (m_connection, name);
	  m_state = STATE_EXECUTING;
	}
    }
  catch (const std::exception& e)
    {
      end();
      throw DatabaseError(e.what());
    }
}

void
transaction::end()
{
  try
    {
      if (m_autocommit == true)
	{
	  if (m_state == STATE_EXECUTING) // abort a running transaction
	    abort();
	  if (m_transaction) // abort() might delete the transaction
	    {
	      delete m_transaction;
	      m_transaction = NULL;
	    }
	  m_state = STATE_NONE;
	}
    }
  catch (const std::exception& e)
    {
      throw DatabaseError(e.what());
    }
}

pqxx::result
transaction::exec(const std::string& query)
{
  try
    {
      return m_transaction->exec(query);
    }
  catch (const std::exception& e)
    {
      throw DatabaseError(e.what());
    }
}

pqxx::result::size_type
transaction::exec_noresult(const std::string& query)
{
  try
    {
      pqxx::result R = exec(query);
      return R.affected_rows();
    }
  catch (const std::exception& e)
    {
      throw DatabaseError(e.what());
    }
}

pqxx::result::size_type
transaction::perform(const std::string& query,
		     pqxx::result::size_type min_rows,
		     pqxx::result::size_type max_rows)
{
  try
    {
      begin("transaction::perform()");

      pqxx::result::size_type rows;
      rows = exec_noresult(query);

      // Commit if the number of altered rows is greater or equal to
      // min_rows, or if min_rows is 0.
      if ((rows >= min_rows || min_rows == 0) &&
	  (rows <= max_rows || max_rows == 0))
	commit();
      else
	abort();

      end();

      return rows;
    }
  catch (const std::exception& e)
    {
      end();
      throw DatabaseError(e.what());
    }

      // This should never be reached.
      return 0;
    }

void
transaction::commit()
{
  try
    {
      if (m_autocommit == true)
	{
	  if (m_autocommit == true)
	    m_transaction->commit();
	}
      m_state = STATE_COMMITTED;
      end();
      m_signal_commit.emit();
    }
  catch (const std::exception& e)
    {
      end();
      throw DatabaseError(e.what());
    }
}

void
transaction::abort()
{
  try
    {
      if (m_autocommit == true)
	{
	  if (m_autocommit == true)
	    m_transaction->abort();
	}
      m_state = STATE_ABORTED;
      end();
      m_signal_abort.emit();
    }
  catch (const std::exception& e)
    {
      end();
      throw DatabaseError(e.what());
    }
}
