// database transaction convenience wrapper                      -*- C++ -*-
// $Id: transaction.h,v 1.5 2004/01/17 23:30:44 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef PQXX_OBJECT_TRANSACTION_H
#define PQXX_OBJECT_TRANSACTION_H

#include <pqxx/connection>
#include <pqxx/transaction>

#include <sigc++/signal.h>

#include <pqxx-object/exceptions.h>

namespace pqxxobject
{
  /**
   * Database transaction convenience wrapper.
   * This class provides methods to simplify database access for
   * classes which perform any sort of database interaction.  Methods
   * are provided to begin and end transactions, and commit and abort
   * transactions.
   *
   * The class provides two modes of operation: auto-commit mode,
   * where transactions are created and destroyed on-the-fly, and
   * non-auto-commit mode, where the user supplies a transaction at
   * construction time, and this is persistent through the lifetime of
   * the object.
   */
  class transaction
  {
  public:
    enum state
      {
	STATE_NONE,
	STATE_ABORTED,
	STATE_COMMITTED,
	STATE_EXECUTING
      };

    /**
     * The constructor.
     * @param connection a reference to a PostgreSQL database
     * connection object.
     * @param transaction a pointer to a PostgreSQL transaction
     * object.  This must have been previously created from
     * connection.  If set to NULL, transaction will run in
     * auto-commit mode, and will create a new transaction for each
     * database operation.
     */
    transaction(pqxx::connection& connection,
		pqxx::transaction<>*& transaction);


    /// The destructor.
    virtual ~transaction();

    /**
     * Begin a new transaction.
     * This method should be called at the start of any database
     * operation.  When running in auto-commit mode, a new
     * pqxx::Transaction object is created.
     *
     * This method has no effect when not running in auto-commit mode.
     * @param name the name to give the transaction.  This may be zero
     * length, but will be shown on error messages relating to the
     * transaction.
     */
    void begin(const std::string& name);

    /**
     * End a transaction.
     * This method should be called at the end of any database
     * operation.  When running in auto-commit mode, the
     * pqxx::Transaction object is deleted.
     *
     * This method has no effect when not running in auto-commit mode.
     */
    void end();

    /**
     * Execute a query.
     * This method should be called to run an SQL statement on the database server.
     * @param query the SQL statement to run.
     * @returns a pointer to the result set.  This is allocated with
     * new and must be deleted when finished with.
     */
    pqxx::result exec(const std::string& query);

    /**
     * Execute a query without seeing the result set.
     * This method should be called to run an SQL statement on the
     * database server.  Unlike exec_query(), this method frees the
     * result set automatically.  Use exec_query if access to the
     * result set is required.
     * @param query the SQL statement to run.
     * @returns the number of rows the query affected.
     */
    pqxx::result::size_type exec_noresult(const std::string& query);

    /**
     * Execute a query and automatically commit or abort the changes.
     * This method should be called to run an SQL statement on the
     * database server.  Unlike exec_query(), this method frees the
     * result set automatically.  Use exec_query if access to the
     * result set is required.  The transaction will be committed or
     * aborted if running in auto-commit mode.
     * @param query the SQL statement to run.
     * @param min_rows the minimum number of rows to be altered.  If
     * the number of rows is greater than or equal to this number, the
     * transaction is committed or else it is aborted.  If set to 0,
     * the operation will always be committed.
     * @param max_rows the maximum number of rows to be altered.  If
     * the number of rows is less that or equal to this number, the
     * transaction is committed or else it is aborted.  If set to 0,
     * the operation will always be committed.
     * @returns the number of rows the query affected.
     */
    pqxx::result::size_type perform(const std::string& query,
				    pqxx::result::size_type min_rows,
				    pqxx::result::size_type max_rows);

    /**
     * Commit a transaction to the database.
     * This method should be called after the successful completion of
     * one, or a series of, successful database operations.  The
     * changes will be committed to the database as one single
     * operation.  After calling this method, the transaction may no
     * longer be used.
     *
     * On commit, the signal_commit signal is emitted.
     *
     * This method has no effect when not running in auto-commit mode.
     */
    void commit();


    /**
     * Abort a transaction.
     * This method should be called after an error, to revert any
     * changes.  The changes will be rolled back to their state at the
     * start of the transaction as a single operation.  After calling
     * this method, the transaction may no longer be used.
     *
     * On abort, the signal_abort signal is emitted.
     *
     * This method has no effect when not running in auto-commit mode.
     */
    void abort();

    SigC::Signal0<void>& signal_commit();
    SigC::Signal0<void>& signal_abort();

    // PostgreSQL connection details
    /// A reference to a PostgreSQL database connection object.
    pqxx::connection& m_connection;
    /// A reference to a pointer to a PostgreSQL transaction object.
    pqxx::transaction<>*& m_transaction;
    /// Set to true if auto-commit mode is in use, or else set to false.
    bool m_autocommit;

    SigC::Signal0<void> m_signal_commit;
    SigC::Signal0<void> m_signal_abort;

    state m_state;

  }; // class transaction

}; // namespace pqxxobject

#endif // PQXX_OBJECT_TRANSACTION_H
