// tutorial places driver program                                -*- C++ -*-
// $Id: places-main.cc,v 1.4 2004/01/17 23:30:46 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <iomanip>
#include <iostream>

#include "places.h"

using namespace pqxx;

void print_header()
{
  std::cout << " id  |        name        | gridref \n"
	    << "-----+--------------------+---------\n";
}

void print_place(const Place& place)
{
  std::cout << std::setw(4) << std::setfill(' ') << std::right << place.get_id() << " | "
	    << std::setw(18) << std::setfill('.') << std::left << place.get_name()
	    << " | " << place.get_gridref() << "\n";
}

void print_list(const PlaceTable::row_list_ptr& list)
{
  print_header();
  for (PlaceTable::row_list::const_iterator cur = list->begin();
       cur != list->end();
       ++cur)
    print_place(*cur);
  std::cout << "(" << list->size() << " rows)\n";
}

int main()
{
  try
    {
      // Create a new database connection.
      std::cout << "Creating a database connection..." << std::flush;
      connection C("dbname=libpqxx_object_tutorial");
      std::cout <<"done.\n";

      // Create a PlaceTable object (represents the places table).
      std::cout << "Creating a PlaceTable..." << std::flush;
      pqxx::transaction<> *T = 0;
      pqxxobject::transaction tran(C, T);
      PlaceTable myplaces(tran);
      std::cout <<"done.\n\n";

      // Clean up database, so this program can be idempotent.
      PlaceTable::row_ptr delplace = myplaces.find_name("Inveraray");
      if (delplace.get() != NULL)
	myplaces.erase(*delplace);

      // Print a list of all the places in the table.
      std::cout << "A list of all places in the places database table.\n";
      PlaceTable::row_list_ptr list = myplaces.get_list();

      print_list(list);

      // Print a list of all the places in the table, sorted by grid reference
      std::cout << "\nA list of all places in the places database table, sorted by grid reference\n";
      list = myplaces.get_list(PlaceTable::ORDER_GRIDREF);

      print_list(list);

      // Find a place by ID.
      std::cout << "\nFinding a place by its ID number.\n";
      PlaceTable::row_ptr place = myplaces.find(4);
      if (place.get() != NULL)
	{
	  print_header();
	  print_place(*place);
	}
      else
	std::cout << "No place found\n";

      // Find a place by name.
      std::cout << "\nFinding a place by its name.\n";
      place = myplaces.find_name("Kirkwall");
      if (place.get() != NULL)
	{
	  print_header();
	  print_place(*place);
	}
      else
	std::cout << "No place found\n";

      // Find a place by grid reference.
      std::cout << "\nFinding a place by its grid reference.\n";
      list = myplaces.find_gridref("SM9537");

      print_list(list);

      // Insert a new place.
      std::cout << "\nInserting a new place: Inveraray, grid reference NB9999\n";
      Place newplace("Inveraray", "NB9999");
      myplaces.insert(newplace);

      // Check the insertion worked.
      std::cout << "\nCheck the insertion worked.\n";
      place = myplaces.find_name("Inveraray");
      if (place.get() != NULL)
	{
	  print_header();
	  print_place(*place);
	}
      else
	std::cout << "No place found\n";

      // Update the place we found previously.
      std::cout << "\nUpdating the row: the grid reference should be NN0908\n";
      if (place.get() != NULL)
	{
	  place->set_gridref("NN0908");
	  myplaces.update(*place);
	}
      else
	std::cout << "No place to update\n";

      // Check the update worked.
      std::cout << "\nCheck the update worked.\n";
      place = myplaces.find_name("Inveraray");
      if (place.get() != NULL)
	{
	  print_header();
	  print_place(*place);
	}
      else
	std::cout << "No place found\n";

      // Now try to delete a place.
      std::cout << "\nNow delete the row.\n";
      if (place.get() != NULL)
	myplaces.erase(*place);
      else
	std::cout << "No place to delete\n";

      // Check the deletion worked.
      std::cout << "\nCheck the deletion worked (no row should be found).\n";
      place = myplaces.find_name("Inveraray");
      if (place.get() != NULL)
	{
	  print_header();
	  print_place(*place);
	}
      else
	std::cout << "No place found\n";
    }
  catch (const pqxxobject::DatabaseError& e)
    {
      std::cerr << "\nCaught a DatabaseError: " << e.what() << std::endl;
      return 1;
    }
  catch (const std::exception& e)
    {
      std::cerr << "\nCaught an exception: " << e.what() << std::endl;
      return 1;
    }
  catch (...)
    {
      std::cerr << "\nCaught an unknown exception; terminating now." << std::endl;
      return 1;
    }
  return 0;
}
