// tutorial places class                                         -*- C++ -*-
// $Id: places.h,v 1.7 2004/01/18 00:11:17 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef TUTORIAL_PLACES_H
#define TUTORIAL_PLACES_H

#include <string>
#include <vector>

#include <pqxx-object/column.h>
#include <pqxx-object/row.h>
#include <pqxx-object/table.h>

/**
 * Place details.
 * This class stores the details of a single place.  It represents a
 * single row in the places database table.  The place id number, name
 * and OS grid reference are known.  Each may be accessed and changed
 * using the class methods, with the exception of setting the id
 * number (which is set by the backend database).
 */
class Place : public pqxxobject::row<Place>
{
public:
  /// The constructor.
  Place();

  /**
   * The constructor.
   * @param name the place name.
   * @param gridref the place grid reference.
   */
  Place(const std::string& name,
	const std::string& gridref);

  /// The destructor.
  virtual ~Place();

  /**
   * Get the place id number.
   * @returns the place id.
   */
  int get_id() const;

  /**
   * Get the place name.
   * @returns a constant reference to the name.
   */
  const std::string& get_name() const;

  /**
   * Set the place name.
   * @param name the place name.
   */
  void set_name(const std::string& name);

  /**
   * Get the place grid reference.
   * @returns a constant reference to the grid reference.
   */
  const std::string& get_gridref() const;

  /**
   * Set the place grid reference.
   * @param gridref the grid reference
   */
  void set_gridref(const std::string& gridref);

  /**
   * Convert a row of an SQL result set into a Place object.
   * @param row an interator pointing to a row of an SQL result set.
   * @param tran a PostgreSQL transaction object.
   * @returns a Place.
   */
  static row_ptr create(pqxx::result::const_iterator row,
			pqxxobject::transaction& tran);

protected:
  /**
   * Insert a row into a table.
   */
  virtual void insert_impl(pqxxobject::transaction& tran);

  /**
   * Update a row from a table.
   */
  virtual void update_impl(pqxxobject::transaction& tran);

  /**
   * Remove a row from a table.
   */
  virtual void erase_impl(pqxxobject::transaction& tran);

  /**
   * Refresh a row from a table.
   */
  virtual void refresh_impl(pqxxobject::transaction& tran);

private:
  /// ID number.
  pqxxobject::column<int> m_id;
  /// Name.
  pqxxobject::column<std::string> m_name;
  /// Grid Reference.
  pqxxobject::column<std::string> m_gridref;


}; // class Place

/**
 * Place database interaction.
 * This class represents the places database table, and may be used to
 * obtain information about places held in the place table.  It may be
 * used to add new places, update information about existing places,
 * and remove places.
 */
class PlaceTable : public pqxxobject::table<Place>
{
public:
  /// Underlying table type.
  typedef pqxxobject::table<Place> table_base;

  /**
   * The constructor.
   * @param tran a PostgreSQL transaction object.
   */
  PlaceTable(pqxxobject::transaction& tran);

  /// The destructor.
  virtual ~PlaceTable();

  /// Sort order
  enum sort_order
    {
      /// Order by ID number.
      ORDER_ID,
      /// Order by name.
      ORDER_NAME,
      /// Order by grid reference.
      ORDER_GRIDREF
    };

  /**
   * Get a list of all places.
   * @param order the order to sort the list in.
   * @returns a pointer to a vector of places.
   */
  row_list_ptr get_list(sort_order order = ORDER_NAME);

  /**
   * Find a single place by its ID number.
   * @param place_id the place ID.
   * @returns a pointer to the place, or NULL if no place was found.
   */
  row_ptr find(int place_id);

  /**
   * Find a place by its name.
   * @param name the place name.
   * @returns a pointer to the place, or NULL if no place was found.
   */
  row_ptr find_name(const std::string& name);

  /**
   * Find places by their grid reference.
   * Because there is no garuantee that the grid reference is
   * unique, this function may return zero, one or many places.
   * @param gridref the place grid reference.
   * @returns a pointer to a vector of places.
   */
  row_list_ptr find_gridref(const std::string& gridref);

}; // class PlaceTable

#endif // TUTORIAL_PLACES_H
